/*
 * This file is part of TONG.              http://www.nongnu.org/tong
 * Copyright 2004 Owen Swerkstrom
 *
 * TONG! is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * TONG! is distributed in the hope that it will be useful and fun,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY of FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TONG!; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>

#include "media.h"
#include "tetris.h"

extern SDL_Surface *back, *zone, *temp, *title, *scorebg, *timebg;
extern SDL_Surface *timeup, *nextbg, *rulebg, *ballbounce, *ballignore;
extern SDL_Surface *balljoin, *balldie, *tetradmove, *tetradignore;
extern SDL_Surface *tetradjoin, *tetradbreak, *tetraddie, *tetradrotate;
extern SDL_Surface *tetraddrop, *stackbreak, *stackignore, *stackjoin;
extern SDL_Surface *stackdie, *stacklinebreak, *gameover, *youlose;
extern SDL_Surface *icon;
extern Mix_Music *music;
extern Mix_Chunk *toing, *newrule, *twobits;
extern Mix_Chunk *rotate1, *rotate2, *plop, *bust, *tick;
extern Mix_Chunk *bounce1, *bounce2;

void draw_bg(bool drawgameboard, SDL_Surface* surf, Option* ops) {
  SDL_Rect dest;
  dest.x=0;
  dest.y=(SDL_GetTicks()/80)%480;
  safe_blit(back, surf, dest);
  dest.x=0;
  dest.y=((SDL_GetTicks()/80)%480)-480;
  safe_blit(back, surf, dest);
  if(drawgameboard) {
    dest.x=224;
    dest.y=64;
    safe_blit(zone, surf, dest);
    dest.x=448;
    dest.y=64;
    safe_blit(scorebg, surf, dest);
    dest.x=64;
    safe_blit(timebg, surf, dest);

    //ball+tetrad rule
    dest.x=64;
    dest.y=144;
    safe_blit(rulebg, surf, dest);
    if(ops->GetRuleBallTetrad()==BALL_JOINS_TETRAD) {
      safe_blit(balljoin, surf, dest);
    } else {
      switch(ops->GetRuleTetradBall()) {
      case NO_REACTION:
	safe_blit(ballignore, surf, dest);
	break;
      case TETRAD_KILLS_BALL:
	safe_blit(balldie, surf, dest);
	break;
      case TETRAD_BOUNCES_BALL:
      default:
	safe_blit(ballbounce, surf, dest);
	break;
      }
    }
    dest.y=208;
    switch(ops->GetRuleBallTetrad()) {
    case NO_REACTION:
      safe_blit(tetradignore, surf, dest);
      break;
    case BALL_ROTATES_TETRAD:
      safe_blit(tetradrotate, surf, dest);
      break;
    case BALL_BREAKS_TETRAD:
      safe_blit(tetradbreak, surf, dest);
      break;
    case BALL_DROPS_TETRAD:
      safe_blit(tetraddrop, surf, dest);
      break;
    case BALL_KILLS_TETRAD:
      safe_blit(tetraddie, surf, dest);
      break;
    case BALL_JOINS_TETRAD:
      safe_blit(tetradjoin, surf, dest);
      break;
    case BALL_MOVES_TETRAD:
    default:
      safe_blit(tetradmove, surf, dest);
      break;
    }

    //ball+stack rule
    dest.y=288;
    safe_blit(rulebg, surf, dest);
    if(ops->GetRuleBallStack()==BALL_JOINS_STACK) {
      safe_blit(balljoin, surf, dest);
    } else {
      switch(ops->GetRuleStackBall()) {
      case NO_REACTION:
	safe_blit(ballignore, surf, dest);
	break;
      case STACK_KILLS_BALL:
	safe_blit(balldie, surf, dest);
	break;
      case STACK_BOUNCES_BALL:
      default:
	safe_blit(ballbounce, surf, dest);
	break;
      }
    }
    dest.y=352;
    switch(ops->GetRuleBallStack()) {
    case NO_REACTION:
      safe_blit(stackignore, surf, dest);
      break;
    case BALL_LINEBREAK_STACK:
      safe_blit(stacklinebreak, surf, dest);
      break;
    case BALL_KILLS_STACK:
      safe_blit(stackdie, surf, dest);
      break;
    case BALL_JOINS_STACK:
      safe_blit(stackjoin, surf, dest);
      break;
    case BALL_BREAKS_STACK:
    default:
      safe_blit(stackbreak, surf, dest);
      break;
    }

    //next tetrad
    dest.x=PREVIEW_OFFSET_X-16;
    dest.y=PREVIEW_OFFSET_Y-16;
    safe_blit(nextbg, surf, dest);
  }
}

void draw_title(SDL_Surface* surf) {
  SDL_Rect dest;
  dest.x=166;
  dest.y=64;
  safe_blit(title, surf, dest);
}

void draw_gameover(SDL_Surface* surf) {
  SDL_Rect dest;
  dest.x=160;
  dest.y=96;
  safe_blit(timeup, surf, dest);
}

void draw_gameover(SDL_Surface* surf, Uint32 timespent) {
  SDL_Rect dest;
  dest.x=160;
  dest.y=96;
  if(timespent<15000) {
    safe_blit(youlose, surf, dest);
  } else {
    safe_blit(gameover, surf, dest);
  }
}

SDL_Surface* init_video(int bpp, int videoflags) {
  if(!SDL_WasInit(SDL_INIT_VIDEO)) {
    SDL_Init(SDL_INIT_VIDEO);
  }
  SDL_Surface* screen;
  screen=SDL_SetVideoMode(640, 480, bpp, videoflags);
  if ( screen == NULL ) {
    printf("Unable to set 640x480 video: %s\n", SDL_GetError());
    exit(1);
  }

  return screen;
}

void init_audio(Option* ops, bool startsong, Uint32& musicstarted) {
  if(music) {
    Mix_FreeMusic(music);
  }
  if(rotate1) {
    Mix_FreeChunk(rotate1);
  }
  if(rotate2) {
    Mix_FreeChunk(rotate2);
  }
  if(plop) {
    Mix_FreeChunk(plop);
  }
  if(bust) {
    Mix_FreeChunk(bust);
  }
  if(tick) {
    Mix_FreeChunk(tick);
  }
  if(bounce1) {
    Mix_FreeChunk(bounce1);
  }
  if(bounce2) {
    Mix_FreeChunk(bounce2);
  }
  if(toing) {
    Mix_FreeChunk(toing);
  }
  if(newrule) {
    Mix_FreeChunk(newrule);
  }
  if(twobits) {
    Mix_FreeChunk(twobits);
  }
  Mix_CloseAudio();
  if(!Mix_OpenAudio(ops->GetAudioFreq(), MIX_DEFAULT_FORMAT, 2,
		    ops->GetAudioBuffers())) {
    if(ops->GetMusic()!=MUSIC_NONE) {  
      if(ops->GetCurrentMusicFile()) {
	if((music=Mix_LoadMUS(ops->GetCurrentMusicFile()))) {
	  if(startsong) {
	    if(ops->GetMusic()==MUSIC_SHUFFLE) {
	      Mix_PlayMusic(music, 1);
	      musicstarted=SDL_GetTicks();
	    } else {
	      Mix_PlayMusic(music, -1);
	      musicstarted=SDL_GetTicks();
	    }
	  } else {
	    if(ops->GetMusic()==MUSIC_SHUFFLE) {
	      Mix_FadeInMusicPos(music, -1, 500,
				 (SDL_GetTicks()-musicstarted)/1000);
	    } else {
	      if(music) {
		Mix_FadeInMusicPos(music, -1, 500,
				   (SDL_GetTicks()-musicstarted)/1000);
	      }
	    }
	  }
	} else {
	  printf("couldn't load music: %s\n", Mix_GetError());
	}
      }
    } else {
      printf("config said no music\n");
    }
    Mix_AllocateChannels(3); //one sfx channels for tetris, one for pong,
                             //one for "new rule" and other such noises
    rotate1=Mix_LoadWAV("media/swish1.ogg");
    if(!rotate1) {
      printf("couldn't load sfx swish1.\n");
    }
    rotate2=Mix_LoadWAV("media/swish2.ogg");
    if(!rotate2) {
      printf("couldn't load sfx swish2.\n");
    }
    plop=Mix_LoadWAV("media/plop1.ogg");
    if(!plop) {
      printf("couldn't load sfx plop.\n");
    }
    bust=Mix_LoadWAV("media/wee1.ogg");
    if(!bust) {
      printf("couldn't load sfx bust.\n");
    }
    tick=Mix_LoadWAV("media/tick.ogg");
    if(!tick) {
      printf("couldn't load sfx tick.\n");
    }
    bounce1=Mix_LoadWAV("media/bounce1.ogg");
    if(!bounce1) {
      printf("couldn't load sfx bounce1.\n");
    }
    bounce2=Mix_LoadWAV("media/bounce2.ogg");
    if(!bounce2) {
      printf("couldn't load sfx bounce2.\n");
    }
    toing=Mix_LoadWAV("media/toing.ogg");
    if(!toing) {
      printf("couldn't load sfx toing.\n");
    }
    newrule=Mix_LoadWAV("media/newrule.ogg");
    if(!newrule) {
      printf("couldn't load sfx newrule.\n");
    }
    twobits=Mix_LoadWAV("media/twobits.ogg");
    if(!twobits) {
      printf("couldn't load sfx twobits.\n");
    }
  } else {
      printf("Couldn't init sound: %s\n", Mix_GetError());
  }
  return;
}

void load_graphics() {
  if(temp) {
    SDL_FreeSurface(temp);
  }
  if(back) {
    SDL_FreeSurface(back);
  }
  if(zone) {
    SDL_FreeSurface(zone);
  }
  if(title) {
    SDL_FreeSurface(title);
  }
  if(scorebg) {
    SDL_FreeSurface(scorebg);
  }
  if(timebg) {
    SDL_FreeSurface(timebg);
  }
  if(rulebg) {
    SDL_FreeSurface(rulebg);
  }
  if(ballbounce) {
    SDL_FreeSurface(ballbounce);
  }
  if(ballignore) {
    SDL_FreeSurface(ballignore);
  }
  if(balljoin) {
    SDL_FreeSurface(balljoin);
  }
  if(balldie) {
    SDL_FreeSurface(balldie);
  }
  if(tetradmove) {
    SDL_FreeSurface(tetradmove);
  }
  if(tetradignore) {
    SDL_FreeSurface(tetradignore);
  }
  if(tetradjoin) {
    SDL_FreeSurface(tetradjoin);
  }
  if(tetradbreak) {
    SDL_FreeSurface(tetradbreak);
  }
  if(tetraddie) {
    SDL_FreeSurface(tetraddie);
  }
  if(tetradrotate) {
    SDL_FreeSurface(tetradrotate);
  }
  if(tetraddrop) {
    SDL_FreeSurface(tetraddrop);
  }
  if(stackbreak) {
    SDL_FreeSurface(stackbreak);
  }
  if(stackignore) {
    SDL_FreeSurface(stackignore);
  }
  if(stackjoin) {
    SDL_FreeSurface(stackjoin);
  }
  if(stackdie) {
    SDL_FreeSurface(stackdie);
  }
  if(stacklinebreak) {
    SDL_FreeSurface(stacklinebreak);
  }
  if(nextbg) {
    SDL_FreeSurface(nextbg);
  }
  if(gameover) {
    SDL_FreeSurface(gameover);
  }
  if(youlose) {
    SDL_FreeSurface(youlose);
  }
  if(icon) {
    SDL_FreeSurface(icon);
  }
  back=read_image("media/background.png");
  zone=read_image("media/playzone.png");
  title=read_image("media/tong.png");
  scorebg=read_image("media/score.png");
  timebg=read_image("media/time.png");
  timeup=read_image("media/timeup.png");
  rulebg=read_image("media/rulespark.png");
  ballbounce=read_image("media/ball-bounce.png");
  ballignore=read_image("media/ball-ignore.png");
  balljoin=read_image("media/ball-join.png");
  balldie=read_image("media/ball-die.png");
  tetradmove=read_image("media/tetrad-move.png");
  tetradignore=read_image("media/tetrad-ignore.png");
  tetradjoin=read_image("media/tetrad-join.png");
  tetradbreak=read_image("media/tetrad-break.png");
  tetraddie=read_image("media/tetrad-die.png");
  tetradrotate=read_image("media/tetrad-rotate.png");
  tetraddrop=read_image("media/tetrad-drop.png");
  stackbreak=read_image("media/stack-break.png");
  stackignore=read_image("media/stack-ignore.png");
  stackjoin=read_image("media/stack-join.png");
  stackdie=read_image("media/stack-die.png");
  stacklinebreak=read_image("media/stack-linebreak.png");
  nextbg=read_image("media/next.png");
  icon=read_image("media/icon.png");
  gameover=read_image("media/gameover.png");
  youlose=read_image("media/youlose.png");
  return;
}

SDL_Surface* read_image(const char* filename) {
  SDL_Surface *temp, *result;
  temp=IMG_Load(filename);
  if(temp) {
    result=SDL_DisplayFormatAlpha(temp);
    SDL_FreeSurface(temp);
  } else {
    printf("Couldn't load image %s.\n", filename);
    result=NULL;
  }
  return result;
}

void safe_blit(SDL_Surface *src, SDL_Surface *dest, SDL_Rect coords) {
  if(SDL_BlitSurface(src, NULL, dest, &coords)) {
    load_graphics();
    SDL_BlitSurface(src, NULL, dest, &coords);
    printf("encountered weirdness drawing to %d,%d\n", coords.x, coords.y);
  }
  return;
}
